@@ -18,7 +18,7 @@ hilight_source = function(x, format, options) {
   } else if (options$prompt) {
     # if you did not reformat or evaluate the code, I have to figure out which
     # lines belong to one complete expression first (#779)
-    if (!options$tidy && isFALSE(options$eval))
+    if (options$engine == 'R' && !options$tidy && isFALSE(options$eval))
       x = vapply(highr:::group_src(x), paste, character(1), collapse = '\n')
     line_prompt(x)
   } else x
