@@ -153,7 +153,9 @@ hook_plot_tex = function(x, options) {
               sub(sprintf('%d$', fig.num), '', sans_ext(x)), 1L, fig.num)
     } else {
       if (nzchar(size)) size = sprintf('[%s]', size)
-      sprintf('\\includegraphics%s{%s} ', size, sans_ext(x))
+      res = sprintf('\\includegraphics%s{%s} ', size, sans_ext(x))
+      lnk = options$fig.link
+      if (is.null(lnk) || is.na(lnk)) res else sprintf('\\href{%s}{%s}', lnk, res)
     },
 
     resize2, sub2, align2, fig2
