@@ -81,7 +81,7 @@ kable = function(
     latex = 'latex', listings = 'latex', sweave = 'latex',
     html = 'html', markdown = 'markdown', rst = 'rst',
     stop('table format not implemented yet!')
-  ) else if (isTRUE(opts_knit$get('bookdown.table.latex')) && is_latex_output()) {
+  ) else if (isTRUE(opts_knit$get('kable.force.latex')) && is_latex_output()) {
     # force LaTeX table because Pandoc's longtable may not work well with floats
     # http://tex.stackexchange.com/q/276699/9128
     'latex'
@@ -191,9 +191,10 @@ kable_latex = function(
   bottomrule = getOption('knitr.table.bottomrule', if (booktabs) '\\bottomrule' else '\\hline'),
   midrule = getOption('knitr.table.midrule', if (booktabs) '\\midrule' else '\\hline'),
   linesep = if (booktabs) c('', '', '', '', '\\addlinespace') else '\\hline',
-  caption = NULL, table.envir = if (!is.null(caption)) 'table', escape = TRUE
+  caption = NULL, caption.short = '', table.envir = if (!is.null(caption)) 'table',
+  escape = TRUE
 ) {
-  if (!is.null(align <- attr(x, 'align', exact = TRUE))) {
+  if (!is.null(align <- attr(x, 'align'))) {
     align = paste(align, collapse = vline)
     align = paste0('{', align, '}')
   }
@@ -206,7 +207,8 @@ kable_latex = function(
   if (identical(caption, NA)) caption = NULL
   env1 = sprintf('\\begin{%s}\n', table.envir)
   env2 = sprintf('\n\\end{%s}',   table.envir)
-  cap = if (is.null(caption)) '' else sprintf('\n\\caption{%s}', caption)
+  if (caption.short != '') caption.short = paste0('[', caption.short, ']')
+  cap = if (is.null(caption)) '' else sprintf('\n\\caption%s{%s}', caption.short, caption)
 
   if (nrow(x) == 0) midrule = ""
 
@@ -247,7 +249,7 @@ kable_html = function(x, table.attr = '', caption = NULL, escape = TRUE, ...) {
   table.attr = gsub('^\\s+|\\s+$', '', table.attr)
   # need a space between <table and attributes
   if (nzchar(table.attr)) table.attr = paste('', table.attr)
-  align = if (is.null(align <- attr(x, 'align', exact = TRUE))) '' else {
+  align = if (is.null(align <- attr(x, 'align'))) '' else {
     sprintf(' style="text-align:%s;"', c(l = 'left', c = 'center', r = 'right')[align])
   }
   if (identical(caption, NA)) caption = NULL
@@ -297,7 +299,7 @@ kable_mark = function(x, sep.row = c('=', '=', '='), sep.col = '  ', padding = 0
     if (grepl('^\\s*$', cn[1L])) cn[1L] = rownames.name  # no empty cells for reST
     l = pmax(if (length(l) == 0) 0 else l, nchar(cn, type = 'width'))
   }
-  align = attr(x, 'align', exact = TRUE)
+  align = attr(x, 'align')
   padding = padding * if (length(align) == 0) 2 else {
     ifelse(align == 'c', 2, 1)
   }
