@@ -31,14 +31,15 @@
 #' attr(res, 'XML')  # all information
 #' if (interactive()) browseURL(res)
 #'
-#' ## to use your own key
+#' # to use your own key
 #' opts_knit$set(upload.fun = function(file) imgur_upload(file, key = 'your imgur key'))
 #' }
 imgur_upload = function(file, key = '9f3460e67f308f6') {
   if (!is.character(key)) stop('The Imgur API Key must be a character string!')
   res = RCurl::postForm(
     'https://api.imgur.com/3/image.xml', image = RCurl::fileUpload(file),
-    .opts = RCurl::curlOptions(httpheader = c(Authorization = paste('Client-ID', key)))
+    .opts = RCurl::curlOptions(httpheader = c(Authorization = paste('Client-ID', key)),
+                               cainfo = system.file("CurlSSL", "cacert.pem", package = "RCurl"))
   )
   res = XML::xmlToList(res)
   if (is.null(res$link)) stop('failed to upload ', file)
